/*jslint nomen: true, indent: 2, maxerr: 3 */
/*global window, rJS, Handlebars, document, loopEventListener, RSVP, URL,
   encodeURIComponent */
(function (window, rJS, jIO) {
  "use strict";
  var DEFAULT_APP_LIST_DOC = "app-list.json",
      SAMPLE_APP_LIST_DOC = "sample-app-list.json";

  function getParameterDict() {
    var hash = window.location.hash.substring(1),
      params = {};
    /*
    hash.split('&').map(hk => {
      let temp = hk.split('=');
      params[temp[0]] = temp[1];
    });
    */
    return params;
  }

  function log(gadget, str, klass) {
    var p = document.createElement('p'),
      old_p_element = gadget.props.element.querySelector(klass + ' p'),
      log_element = gadget.props.element.querySelector(klass);
    if (old_p_element) {
      old_p_element.remove();
    }
    p.innerHTML = str;
    log_element.style.display = "";
    log_element.appendChild(p);
  }

  function logAddApp(gadget, str) {
    return log(gadget, str, ".crib-add-app-status");
  }

  function putNewAppList(gadget, app_list) {
    /*
    *   Add a app in the app list. As the app list is hosted directly on this
    * url, we switch back and forth with the current edit site URL in order to
    * to set it right.
    *
    */
    var current_edit_url,
        this_edit_url = new URL(
          "./crib-enable.html", window.location.href).href;
    return RSVP.Queue()
      .push(function () {
        return gadget.crib_sw_getCribEnableGadgetUrl();
      })
      .push(function (url) {
        current_edit_url = url;
        return gadget.crib_sw_setCribEnableGadgetUrl(this_edit_url);
      })
      .push(function () {
        return gadget.crib_sw_put(DEFAULT_APP_LIST_DOC, {
          content: JSON.stringify(app_list),
          type: "application/json"
        });
      })
      .push(function () {
        return gadget.crib_sw_setCribEnableGadgetUrl(current_edit_url);
      }, function () {
        return gadget.crib_sw_setCribEnableGadgetUrl(current_edit_url);
      });
  }

  function getApplicationList(gadget) {
    return RSVP.Queue()
      .push(function () {
        return gadget.fetch(DEFAULT_APP_LIST_DOC)
          .push(undefined, function () {
            return gadget.fetch(SAMPLE_APP_LIST_DOC);
          });
      })
      .push(function (data) {
        return JSON.parse(data);
      });
  }

  function loadCribJSFromZip(gadget, event) {
    var app_list = [],
      to_path = gadget.props
        .element.querySelector("form.crib-load-from-zip .load-zip-to-path").value + "/",
      redirect_to = to_path +
        gadget.props.element.querySelector("form.crib-load-from-zip .redirect-url").value;
    if (gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-file").files.length === 0 &&
        gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-url").value.length === 0) {
      logAddApp(gadget, "Please set a Zip File or Zip Url");
      return;
    }
    return RSVP.Queue()
      .push(function () {
        var promise_list = [];
        if (gadget.props.element
              .querySelector("form.crib-load-from-zip .load-zip-file").files.length !== 0) {
          logAddApp(gadget, "Will load Zip from File");
          promise_list.push(loadCribJSFromZipFile(gadget, event, logAddApp));
        } else {
          logAddApp(gadget, "Will load Zip from Url");
          promise_list.push(loadCribJSFromZipUrl(gadget, event, logAddApp));
        }
        promise_list.push(getApplicationList(gadget));
        return RSVP.all(promise_list);
      })
      .push(function (result_list) {
        app_list = result_list[1];
        app_list.splice(
          0, 0, {
            int_index: 41,
            application_image_type: undefined,
            application_image_url: undefined,
            application_description: undefined,
            application_url: redirect_to,
            application_path: to_path,
            application_edit_page: redirect_to,
            application_title: gadget.props.element.querySelector(".app-name").value,
            application_category: undefined
          }
        );
        logAddApp(gadget, "Adding App to App list");
        return putNewAppList(gadget, app_list);
      })
      .push(function () {
        logAddApp(gadget, "Redirecting to Editor");
        return gadget.redirect({page: "editor", url: redirect_to});
      });
  }

  function loadCribJSFromZipFile(gadget, event, logFunction) {
    return RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function (jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipFile({
        path: document.location.href,
        file_list: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-file").files,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        to_path: gadget.props
          .element.querySelector("form.crib-load-from-zip .load-zip-to-path").value,
        application_id: "cribjs"
      },
      function (str) {
        logFunction(gadget, str);
      });
    });
  }

  function loadCribJSFromZipUrl(gadget, event, logFunction) {
    return RSVP.Queue()
    .push(function () {
      return gadget.getDeclaredGadget("jio_cribjs");
    })
    .push(function (jio_cribjs_gadget) {
      return jio_cribjs_gadget.loadFromZipUrl({
        path: document.location.href,
        zip_url: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-url").value,
        from_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-from-zip-path").value,
        to_path: gadget.props.element
          .querySelector("form.crib-load-from-zip .load-zip-to-path").value,
        application_id: "cribjs"
      },
      function (str) {
        logFunction(gadget, str);
      });
    });
  }

  rJS(window)
    /////////////////////////////////////////////////////////////////
    // ready
    /////////////////////////////////////////////////////////////////

    // Assign the element to a variable
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    //////////////////////////////////////////////
    // acquired methods
    //////////////////////////////////////////////
    .declareAcquiredMethod("redirect", "redirect")
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .declareAcquiredMethod(
      "crib_sw_setCribEnableGadgetUrl",
      "crib_sw_setCribEnableGadgetUrl"
    )
    .declareAcquiredMethod(
      "crib_sw_getCribEnableGadgetUrl",
      "crib_sw_getCribEnableGadgetUrl"
    )
    .allowPublicAcquisition("crib_sw_get", function (argument_list) {
      return this.crib_sw_get(argument_list[0]);
    })
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .allowPublicAcquisition("crib_sw_put", function (argument_list) {
      return this.crib_sw_put(argument_list[0], argument_list[1]);
    })
    .declareMethod('render', function (options) {
      var gadget = this,
        getURL = window.location,
        site = getURL.protocol + "//" + getURL.host,
        params = getParameterDict();
      if (params.hasOwnProperty("from_path")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .load-from-zip-path").value = params.from_path;
      }
      if (params.hasOwnProperty("to_path")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .load-zip-to-path").value = params.to_path;
      }
      if (params.hasOwnProperty("zip_url")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .load-zip-url").value = params.zip_url;
      }
      if (params.hasOwnProperty("redirect_url")) {
        gadget.props.element.querySelector("form.crib-load-from-zip .redirect-url").value = params.redirect_url;
      }
      return gadget.props.start_deferred.resolve();
    })
    .declareMethod('fetch', function (url) {
      var context = this;
      return new RSVP.Queue()
        .push(function () {
          return jIO.util.ajax({
            url: url
          });
        })
        .push(function (evt) {
          return evt.target.responseText;
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to get content from URL
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector("form.crib-load-from-zip"),
            'submit',
            false,
            function (event) {loadCribJSFromZip(gadget, event); }
          ));
          return RSVP.all(promise_list);
        });
    });



}(window, rJS, jIO));