/*global window, rJS, loopEventListener, RSVP, console, document, WebTorrent,
  File, setInterval, clearInterval, mimeType, URL*/
/*jslint nomen: true, indent: 2, maxerr: 3*/
(function (window, rJS, loopEventListener, JSZip, WebTorrent,
           mimeType) {
  "use strict";

  function log(gadget, str, klass) {
    var p = document.createElement('p'),
      old_p_element = gadget.props.element.querySelector(klass + ' p'),
      log_element = gadget.props.element.querySelector(klass);
    if (old_p_element) {
      old_p_element.remove();
    }
    p.innerHTML = str;
    log_element.style.display = "";
    log_element.appendChild(p);
  }

  function logExportZipFile(gadget, str) {
    return log(gadget, str, ".crib-save-to-zip-status");
  }

  function logImportZipFile(gadget, str) {
    return log(gadget, str, ".crib-load-from-zip-status");
  }

  function logExportZipTorrent(gadget, str) {
    return log(gadget, str, ".crib-export-to-zip-torrent-status");
  }

  function logImportZipTorrent(gadget, str) {
    return log(gadget, str, ".crib-load-from-zip-torrent-status");
  }

  function downloadTorrent(gadget, torrentId) {
    logImportZipTorrent(gadget, "Preparing");
    return RSVP.Promise(function (resolve, reject) {
      var client = new WebTorrent();
      client.on('error', function (err) {
        reject(err.message);
      });
      client.add(torrentId, function (torrent) {
        logImportZipTorrent(gadget, 'Got torrent metadata!');
        logImportZipTorrent(gadget,
          'Torrent info hash: ' + torrent.infoHash + ' ' +
          '<a href="' + torrent.magnetURI + '" target="_blank">[Magnet URI]</a> ' +
          '<a href="' + torrent.torrentFileBlobURL + '" target="_blank" download="' + torrent.name + '.torrent">[Download .torrent]</a>'
        );
        // Torrents can contain many files. Here we suppose there is only one zip
        var interval = setInterval(function () {
          logImportZipTorrent(gadget, 'Progress: ' + (torrent.progress * 100).toFixed(1) + '%');
        }, 5000);
        torrent.files[0].getBlob(function (error, result) {
          if (error) {
            reject(error);
          }
          resolve(result);
        });
        torrent.on('done', function () {
          logImportZipTorrent(gadget, "100%");
          clearInterval(interval);
        });

      });

    });
  }

  // Zip Methods
  function prepareZip(gadget, event, options, path_to_save, logFunction) {
    var path_to_save_length, crib_sw_gadget,
      jio_gadget, url_list = [], final_url_list = [], saved_number = 0, zip;
    if (options === undefined) {
      options = {};
    }
    if (options.type === undefined) {
      options.type = "blob";
    }
    path_to_save_length = path_to_save.length;
    logFunction(gadget, "Preparing");
    return new RSVP.Queue()
    .push(function () {
      return RSVP.all([
        gadget.getDeclaredGadget('jio_gadget')
      ]);
    })
    .push(function (gadget_list) {
      jio_gadget = gadget_list[0];
      return gadget.crib_sw_allDocs({cached_only: true});
    })
    .push(function (data) {
      var promise_list = [],
        i, i_len, url;
      if (data.hasOwnProperty("urls")) {
        data = data.urls;
      } else {
        data = data;
      }
      if (Array.isArray(data)) {
        url_list = data;
      } else {
        url_list = Object.keys(data);
      }
      for (i = 0, i_len = url_list.length; i < i_len; i += 1) {
        url = String(url_list[i]);
        if (url.indexOf(path_to_save) === 0) {
          saved_number += 1;
          promise_list.push(gadget.crib_sw_get(url));
          final_url_list.push(url);
        }
      }
      logFunction(gadget, "Retrieving " + saved_number + " Files");
      return RSVP.all(promise_list);
    })
    .push(function (response_list) {
      var promise_list = [],
        i, i_len, url, response, extension, zip;
      zip = new JSZip();
      for (i = 0, i_len = response_list.length; i < i_len; i += 1) {
        response = response_list[i];
        url = final_url_list[i].substr(path_to_save_length);
        if (url.endsWith("//")) {
          url = url.substr(0, url.length - 1);
        }
        if (url.endsWith("/./")) {
          url = url.substr(0, url.length - 2);
        }
        if (url.endsWith("/")) {
          url = url + "index.html";
        }
        if (url.startsWith("./")) {
          url = url.substr(1);
        }
        zip.file(url, response);
      }
      logFunction(gadget, "Preparing Zip");
      return zip.generateAsync({type: options.type});
    });
  }

  function formatSizeUnits(bytes) {
    if      (bytes >= 1073741824) { bytes = (bytes / 1073741824).toFixed(2) + " GB"; }
    else if (bytes >= 1048576)    { bytes = (bytes / 1048576).toFixed(2) + " MB"; }
    else if (bytes >= 1024)       { bytes = (bytes / 1024).toFixed(2) + " KB"; }
    else if (bytes > 1)           { bytes = bytes + " bytes"; }
    else if (bytes == 1)          { bytes = bytes + " byte"; }
    else                          { bytes = "0 bytes"; }
    return bytes;
  }

  function saveContentToZIP(gadget, event) {
    var path_to_save = gadget.props.element
          .querySelector('form.crib-save-to-zip .save-zip-path').value;
    gadget.props.element.querySelector(".crib-save-to-zip-link").style.display = "none";
    return prepareZip(gadget, event, {}, path_to_save, logExportZipFile)
      .push(function (content) {
        logExportZipFile(
          gadget,
          "Saved a " + formatSizeUnits(content.size) + " zip file at " + Date()
        );
        gadget.props.element.querySelector(".crib-save-to-zip-link a")
          .href = URL.createObjectURL(content);
        gadget.props.element.querySelector(".crib-save-to-zip-link").style.display = "";
      })
      .push(console.log, console.log);
  }

  function seedZip(gadget, event) {
    var path_to_save = gadget.props.element
          .querySelector('form.crib-export-torrent .torrent-save-zip-path')
          .value;
    return prepareZip(gadget, event, {}, path_to_save, logExportZipTorrent)
      .push(function (result) {
        logExportZipTorrent(
          gadget,
          "Sharing a " + formatSizeUnits(result.size) + " zip file at " + Date()
        );
        gadget.client = new WebTorrent();
        gadget.client.seed(new File([result], "data.zip"), function (torrent) {
          gadget.props.element.querySelector("div.magnet-link").style.display = "";
          gadget.props.element
            .querySelector("textarea.magnet-link").textContent = torrent.magnetURI;
          console.log('Client is seeding ' + torrent.magnetURI);
        });
      });
  }

  function loadContentFromZIPFile(gadget, event) {
    var file_list, file, path_to_load;
    file_list = gadget.props.element.querySelector('form.crib-load-from-zip .load-zip-file').files;
    path_to_load = gadget.props.element
      .querySelector('form.crib-load-from-zip .load-zip-path').value;
    if (file_list.length === 0) {
      logImportZipFile(gadget, "Please put a Zip at " + Date());
      return;
    }
    file = file_list[0];
    return loadContentFromZIP(
      gadget, event, file, path_to_load, logImportZipFile
    );
  }

  function loadContentFromZIPMagnet(gadget, event) {
    var magnet, path_to_load;
    magnet = gadget.props.element.querySelector('form.crib-import-torrent .zip-magnet').value;
    path_to_load = gadget.props.element
      .querySelector('form.crib-import-torrent .torrent-load-zip-path').value;
    return RSVP.Queue()
      .push(function () {
        return downloadTorrent(gadget, magnet);
      })
      .push(function (result) {
        return loadContentFromZIP(
          gadget, event, result, path_to_load, logImportZipTorrent
        );
      })
      .push(console.log, console.log);

  }

  function loadContentFromZIP(gadget, event, zip_data,
                               path_to_load, logFunction) {
    var path_to_load_length, file_list, crib_sw_gadget,
      jio_gadget, url_list = [], file, url_number = 0;
    path_to_load_length = path_to_load.length;
    return new RSVP.Queue()
    .push(function () {
      return logFunction(gadget, "Loading Zip");
    })
    .push(function () {
      return JSZip.loadAsync(zip_data);
    })
    .push(function (zip) {
      var promise_list = [];
      zip.forEach(function (relativePath, zipEntry) {
        var end_url;
        if (zipEntry.dir) {
          return;
        }
        if (path_to_load && !relativePath.startsWith("/") &&
            !path_to_load.endsWith("/")) {
          end_url = path_to_load + "/" + relativePath;
        } else if (relativePath.startsWith("/") &&
                   (path_to_load.endsWith("/") || path_to_load === "")) {
          end_url = path_to_load + relativePath.substring(1);
        } else {
          end_url = path_to_load + relativePath;
        }
        url_number += 1;
        promise_list.push(
          new RSVP.Queue()
            .push(function () {
              return zipEntry.async('blob');
            })
            .push(function (result) {
              var guessed_type = mimeType.lookup(
                end_url, false, "application/octet-stream"
              );
              result = result.slice(0, result.size, guessed_type);
              return gadget.crib_sw_put(end_url, {blob: result});
            })
        );
      });
      logFunction(
        gadget,
        "Pushing " + url_number + " files in Crib at " + Date()
      );
      return RSVP.all(promise_list);
    })
    .push(function () {
      logFunction(gadget, "Loaded " + url_number + " files at " + Date());
    }, function (e) {
      console.error(e);
      logFunction(gadget, "ERROR:</br>" + e.toString());
    });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return g.getElement()
        .push(function (element) {
          g.props.element = element;
          g.props.start_deferred = RSVP.defer();
        });
    })
    .declareAcquiredMethod("crib_sw_allDocs", "crib_sw_allDocs")
    .declareAcquiredMethod("crib_sw_get", "crib_sw_get")
    .declareAcquiredMethod("crib_sw_put", "crib_sw_put")
    .declareAcquiredMethod("getUrlFor", "getUrlFor")
    .declareMethod('render', function (options) {
      var gadget = this;
      if (options === undefined)
        options = {};
      gadget.props.options = options;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.resolve();
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          if (gadget.props.options.zip_download_path) {
            gadget.props.element
              .querySelector('form.crib-save-to-zip .save-zip-path')
              .value = gadget.props.options.zip_download_path;
          }
          if (gadget.props.options.start_zip_download) {
            return saveContentToZIP(gadget, undefined);
          }
        });
    })
    .declareService(function () {
      var gadget = this;
      return new RSVP.Queue()
        .push(function () {
          return gadget.props.start_deferred.promise;
        })
        .push(function () {
          var promise_list = [];
          // promise to save content to ZIP
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-save-to-zip"),
            'submit',
            false,
            function (event) {saveContentToZIP(gadget, event); }
          ));
          // promise to load content from ZIP
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-load-from-zip"),
            'submit',
            false,
            function (event) {loadContentFromZIPFile(gadget, event); }
          ));
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-export-torrent"),
            'submit',
            false,
            function (event) {seedZip(gadget, event); }
          ));
          promise_list.push(loopEventListener(
            gadget.props.element.querySelector(".crib-import-torrent"),
            'submit',
            false,
            function (event) {loadContentFromZIPMagnet(gadget, event); }
          ));
          return RSVP.all(promise_list);
        });
    });
}(window, rJS, loopEventListener, JSZip, WebTorrent, mimeType));